@@ -31,14 +31,19 @@ public:
 
     ThumbnailCache(const ThumbnailCache &t) = delete;
     ThumbnailCache & operator=(const ThumbnailCache &t) = delete;
+    ThumbnailCache(ThumbnailCache &&t) = delete;
+    ThumbnailCache & operator=(ThumbnailCache &&t) = delete;
 
     std::string get_if_exists(const std::string &abs_path, ThumbnailSize desired_size) const;
     std::string get_cache_file_name(const std::string &as_path, ThumbnailSize desired_size) const;
+    std::string get_fail_file_name(const std::string &as_path) const;
+    void mark_failure(const std::string &abs_path) noexcept;
+    bool has_failure(const std::string &abs_path) const;
     void clear();
     void prune();
 
 private:
-    ThumbnailCachePrivate *p;
+    ThumbnailCachePrivate *const p;
 };
 
 
