@@ -23,10 +23,13 @@
 #include <QDBusInterface>
 #include <QQuickImageProvider>
 
+#include <memory>
+
 class AlbumArtGenerator: public QQuickImageProvider
 {
 private:
-    QDBusInterface iface;
+    std::unique_ptr<QDBusConnection> connection;
+    std::unique_ptr<QDBusInterface> iface;
 
 public:
     AlbumArtGenerator();
