@@ -29,6 +29,11 @@ private:
 
 public:
     ThumbnailGenerator();
+    ThumbnailGenerator(const ThumbnailGenerator &other) = delete;
+    const ThumbnailGenerator & operator=(const ThumbnailGenerator &other) = delete;
+    ThumbnailGenerator(ThumbnailGenerator &&other) = delete;
+    const ThumbnailGenerator & operator=(ThumbnailGenerator &&other) = delete;
+
     QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize);
     QImage getFallbackImage(const QString &id, QSize *size, const QSize &requestedSize);
 };
