@@ -23,8 +23,10 @@
 #include<gdk-pixbuf/gdk-pixbuf.h>
 
 #define TESTIMAGE TESTDATADIR "/testimage.jpg"
+#define TESTIMAGE_NOEXIF TESTDATADIR "/testimage_noexif.png"
 #define ROTTESTIMAGE TESTDATADIR "/testrotate.jpg"
 #define TESTVIDEO TESTDATADIR "/testvideo.ogg"
+#define CHINESETEXTDIR TESTDATADIR "/chinese_text"
 
 using namespace std;
 
@@ -80,6 +82,12 @@ TEST(Thumbnailer, image) {
     file_test(tn, imfile);
 }
 
+TEST(Thumbnailer, noexif_image) {
+    Thumbnailer tn;
+    string imfile(TESTIMAGE_NOEXIF);
+    file_test(tn, imfile);
+}
+
 TEST(Thumbnailer, video) {
     Thumbnailer tn;
     string videofile(TESTVIDEO);
@@ -167,17 +175,38 @@ TEST(Thumbnailer, no_image_cache) {
     ASSERT_EQ(srcimg, dstimg);
 }
 
-TEST(Thumbnailer, album_art) {
+TEST(Thumbnailer, album_and_artist_art) {
     // During test suite run we may not have access to the net.
     // So just test calling to ensure symbol visibility etc work.
     Thumbnailer tn;
     string artist("The Prodigy");
     string album("Music for the Jilted Generation");
     tn.get_album_art(artist, album, TN_SIZE_ORIGINAL, TN_LOCAL);
+    tn.get_artist_art(artist, album, TN_SIZE_ORIGINAL, TN_LOCAL);
+}
+
+TEST(Thumbnailer, chinese_text) {
+    Thumbnailer tn;
+    DIR *dir;
+    struct dirent *ent;
+    char path[PATH_MAX];
+    struct stat buffer;
+    int status;
+
+    dir = opendir(CHINESETEXTDIR);
+    while ((ent = readdir(dir)) != NULL) {
+        sprintf(path, "%s/%s", CHINESETEXTDIR, ent->d_name);
+        status = lstat(path, &buffer);
+        ASSERT_EQ(status, 0);
+        if(!S_ISDIR(buffer.st_mode)) {
+            string srcimg(path);
+            string thumbfile = tn.get_thumbnail(srcimg, TN_SIZE_SMALL);
+            ASSERT_TRUE(file_exists(thumbfile));
+        }
+    }
 }
 
 int main(int argc, char **argv) {
-    g_type_init(); // Still needed in precise.
     ::testing::InitGoogleTest(&argc, argv);
     return RUN_ALL_TESTS();
 }
